import asyncio
import websockets
import json

exg = None
sym = None
msg = None
ws = None


def get(ws, exchange, symbol, isShortIdentifier):
    exg = exchange
    sym = symbol
    isi = isShortIdentifier
    rmsg = asyncio.get_event_loop().run_until_complete(mass_subscribe_n_stream(ws, exg, sym, isi))
    return rmsg


async def mass_subscribe_n_stream(ws, exg, sym, isi):
    try:
        req_msg = str(
            '{"MessageType":"GetLastQuoteShort","Exchange":"' + exg + '","isShortIdentifier":"' + isi + '","InstrumentIdentifier":"' + sym + '"}')
        print("Request : " + req_msg)
        await ws.send(req_msg)
        rmsg = await get_msg(ws)
        return rmsg
    except:
        return msg


async def get_msg(ws):
    while True:
        try:
            message = await ws.recv()
            rslt = json.loads(message)
            if rslt["MessageType"] == 'LastQuoteShortResult':
                return message
        except websockets.ConnectionClosedOK:
            break


import asyncio
import websockets
import json

url = None
key = None
exg = None
sym = None
msg = None
prc = None
prd = None
iss = None


def get(ws, exchange, symbol, periodicity, period,isShortIdentifiers):
    exg = exchange
    sym = symbol
    prc = periodicity
    prd = period
    iss = isShortIdentifiers
    rmsg = asyncio.get_event_loop().run_until_complete(mass_subscribe_n_stream(ws, exg, sym, prc, prd, iss))
    return rmsg


async def mass_subscribe_n_stream(ws, exg, sym, prc, prd, iss):
    try:
        # Format symbols as [{"Value": "symbol"}]
        instrument_identifiers = [{"Value": s} for s in sym.split(',')]

        # Prepare the request message as a dictionary
        req_msg = {
            "MessageType": "GetSnapshot",
            "Exchange": exg,
            "Periodicity": prc,
            "Period": prd,
            "isShortIdentifiers": iss,
            "InstrumentIdentifiers": instrument_identifiers
        }

        # Convert to JSON string for sending
        req_msg_str = json.dumps(req_msg)
        print("Request : " + req_msg_str)

        # Send the request message over the WebSocket
        await ws.send(req_msg_str)

        # Listen for the response message
        rmsg = await get_msg(ws)
        return rmsg
    except Exception as e:
        print(f"An error occurred: {e}")
        return msg


async def get_msg(ws):
    while True:
        try:
            message = await ws.recv()
            rslt = json.loads(message)
            if rslt["MessageType"] == 'SnapshotResult':
                return message
        except websockets.ConnectionClosedOK:
            break



import asyncio
import websockets
import json

url = None
key = None
exg = None
sym = None
msg = None
printed_once = False

def get(ws, exchange, symbol):
    exg = exchange
    sym = symbol
    rmsg = asyncio.get_event_loop().run_until_complete(mass_subscribe_n_stream(ws, exg, sym))
    return rmsg


def stop(ws, exchange, symbol):
    exg = exchange
    sym = symbol
    rmsg = asyncio.get_event_loop().run_until_complete(mass_unsubscribe_n_stream(ws, exg, sym))
    return rmsg


async def mass_subscribe_n_stream(ws, exg, sym):
    global printed_once
    req_msg = str(
        '{"MessageType":"SubscribeRealtime","Exchange":"' + exg + '","Unsubscribe":"false","InstrumentIdentifier":"' + sym + '"}')
    if not printed_once:
            print("Request : " + req_msg)
            printed_once = True
    await ws.send(req_msg)
    rmsg = await get_msg(ws)
    return rmsg


async def mass_unsubscribe_n_stream(ws, exg, sym):
    req_msg = str(
        '{"MessageType":"SubscribeRealtime","Exchange":"' + exg + '","Unsubscribe":"true","InstrumentIdentifier":"' + sym + '"}')
    print("Request : " + req_msg)
    await ws.send(req_msg)
    rmsg = await get_msg(ws)
    return rmsg


async def get_msg(ws):
    while True:
        try:
            message = await ws.recv()
            rslt = json.loads(message)
            if rslt["MessageType"] == 'RealtimeResult':
                return message
        except websockets.ConnectionClosedOK:
            break



# import asyncio
# import websockets
# import json
#
# exg = None
# sym = None
# msg = None
# isi = None
#
#
# def get(ws, exchange, symbols,isShortIdentifiers):
#     exg = exchange
#     sym = symbols
#     isi = isShortIdentifiers
#     rmsg = asyncio.get_event_loop().run_until_complete(mass_subscribe_n_stream(ws, exg, sym, isi))
#     return rmsg
#
#
# async def mass_subscribe_n_stream(ws, exg, sym, isi):
#     try:
#         req_msg = str('{"MessageType":"GetLastQuoteArray","Exchange":"' + exg + '","isShortIdentifiers":"' + isi + '","InstrumentIdentifiers":' + str(sym) + '}')
#         print("Request : " + req_msg)
#         await ws.send(req_msg)
#         rmsg = await get_msg(ws)
#         return rmsg
#     except:
#         return msg
#
#
# async def get_msg(ws):
#     while True:
#         try:
#             message = await ws.recv()
#             rslt = json.loads(message)
#             if rslt["MessageType"] == 'LastQuoteArrayResult':
#                 return message
#         except websockets.ConnectionClosedOK:
#             break
#         print(message)
import asyncio
import websockets
import json

exg = None
sym = None
msg = None
isi = None


def get(ws, exchange, symbols, isShortIdentifiers):
    exg = exchange
    sym = symbols
    isi = isShortIdentifiers
    rmsg = asyncio.get_event_loop().run_until_complete(mass_subscribe_n_stream(ws, exg, sym, isi))
    return rmsg


async def mass_subscribe_n_stream(ws, exg, sym, isi):
    try:
        # Prepare the symbol list as [{"Value": "SYMBOL"}]
        instrument_identifiers = [{"Value": s} for s in sym.split(',')]

        # Construct the request message
        req_msg = {
            "MessageType": "GetLastQuoteArray",
            "Exchange": exg,
            "isShortIdentifiers": isi,
            "InstrumentIdentifiers": instrument_identifiers
        }

        # Convert to JSON string for sending
        req_msg_str = json.dumps(req_msg)
        print("Request : " + req_msg_str)

        # Send the request message over the WebSocket
        await ws.send(req_msg_str)

        # Receive and return the response message
        rmsg = await get_msg(ws)
        return rmsg
    except Exception as e:
        print(f"An error occurred: {e}")
        return msg


async def get_msg(ws):
    while True:
        try:
            message = await ws.recv()
            rslt = json.loads(message)
            if rslt["MessageType"] == 'LastQuoteArrayResult':
                return message
        except websockets.ConnectionClosedOK:
            break
        print(message)






import asyncio
import websockets
import json
import errno
import os

url = None
key = None
exg = None


def get(ws, exchange):
    if exchange == "":
        print("Exchange is mandatory.")
    else:
        exg = exchange

    rmsg = asyncio.get_event_loop().run_until_complete(mass_subscribe_n_stream(ws, exg))
    return rmsg


async def mass_subscribe_n_stream(ws, exg):
    try:
        # req_msg = str(
        #     '{"MessageType":"GetExchangeMessages","Exchange":"' + exg + '"}')
        req_msg = json.dumps({
            "MessageType": "GetExchangeMessages",
            "Exchange": exg
        })
        await ws.send(req_msg)
        print("Request : " + req_msg)
        rmsg = await get_msg(ws)
        return rmsg
    except:
        return "Error"


async def get_msg(ws):
    while True:
        try:
            message = await ws.recv()
            rslt = json.loads(message)
            if rslt["MessageType"] == 'ExchangeMessagesResult':
                return message
        except websockets.ConnectionClosedOK:
            break
        # print(message)



# import asyncio
# import websockets
# import json
#
# url = None
# key = None
# exg = None
# sym = None
# msg = None
# prc = None
# prd = None
# frm = None
# to = None
# iss = None
# rno = None
# utg = None
#
#
# def getbyperiod(ws, exchange, InstrumentIdentifier, periodicity, period, fromtime, totime, usertag, isShortIdentifiers):
#     if exchange is None:
#         return "Exchange is mandatory."
#     else:
#         exg = exchange
#
#     if InstrumentIdentifier is None:
#         return "InstrumentIdentifier / Symbol is mandatory."
#     else:
#         sym = InstrumentIdentifier
#
#     if periodicity is None:
#         prc = 'MINUTE'
#     else:
#         prc = periodicity
#
#     if period is None:
#         prd = '1'
#     else:
#         prd = period
#
#     rmsg = asyncio.get_event_loop().run_until_complete(
#         mass_subscribe_n_streamft(ws, exg, sym, prc, prd, fromtime, totime, isShortIdentifiers, usertag))
#     return rmsg
#
#
# def getcaldle(ws, exchange, InstrumentIdentifier, periodicity, period, max, usertag, isShortIdentifiers):
#     if exchange is None:
#         return "Exchange is mandatory."
#     else:
#         exg = exchange
#
#     if InstrumentIdentifier is None:
#         return "Symbol is mandatory."
#     else:
#         sym = InstrumentIdentifier
#
#     if periodicity is None:
#         prc = 'MINUTE'
#     else:
#         prc = periodicity
#
#     if period is None:
#         prd = '1'
#     else:
#         prd = period
#
#     if max is None:
#         rno = max
#     else:
#         rno = 10
#
#     if isShortIdentifiers == "":
#         iss = 'false'
#     else:
#         iss = isShortIdentifiers
#
#     if usertag is None:
#         utg = ''
#     else:
#         utg = usertag
#
#     rmsg = asyncio.get_event_loop().run_until_complete(
#         mass_subscribe_n_stream(ws, exg, sym, prc, prd, iss, rno, utg))
#     return rmsg
#
#
# async def mass_subscribe_n_streamft(ws, exg, sym, prc, prd, frm, to, iss, utg):
#     try:
#         if iss != "" and utg != "":
#             req_msg = '{"MessageType":"GetHistory","Exchange":"' + exg + '","InstrumentIdentifier":"' + sym + '","Periodicity":"' + prc + '","Period":' + str(
#                 prd) + ',"From":"' + str(frm) + '","To":"' + str(to) + '","isShortIdentifier":"' + iss + '","UserTag":"' + utg + '"}'
#         elif iss == "" and utg != "":
#             req_msg = '{"MessageType":"GetHistory","Exchange":"' + exg + '","InstrumentIdentifier":"' + sym + '","Periodicity":"' + prc + '","Period":' + str(
#                 prd) + ',"From":"' + str(frm) + '","To":"' + str(to) + '","isShortIdentifier":"false","UserTag":"' + utg + '"}'
#         elif iss == "" and utg == "":
#             req_msg = '{"MessageType":"GetHistory","Exchange":"' + exg + '","InstrumentIdentifier":"' + sym + '","Periodicity":"' + prc + '","Period":' + str(
#                 prd) + ',"From":"' + str(frm) + '","To":"' + str(to) + '","isShortIdentifier":"false"}'
#         elif iss != "" and utg == "":
#             req_msg = '{"MessageType":"GetHistory","Exchange":"' + exg + '","InstrumentIdentifier":"' + sym + '","Periodicity":"' + prc + '","Period":' + str(
#                 prd) + ',"From":"' + str(frm) + '","To":"' + str(to) + '","isShortIdentifier":"' + iss + '"}'
#         await ws.send(req_msg)
#         print("Request : " + req_msg)
#         rmsg = await get_msg(ws)
#         return rmsg
#     except:
#         return msg
#
#
# async def mass_subscribe_n_stream(ws, exg, sym, prc, prd, iss, rno, utg):
#     req_msg = None
#     try:
#         if iss != "" and utg != "":
#             req_msg = '{"MessageType":"GetHistory","Exchange":"' + exg + '","InstrumentIdentifier":"' + sym + '","Periodicity":"' + prc + '","Period":' + str(
#                 prd) + ',"Max":' + str(rno) + ',"isShortIdentifier":"' + iss + '","UserTag":"' + utg + '"} '
#         elif iss != "" and utg == "":
#             req_msg = '{"MessageType":"GetHistory","Exchange":"' + exg + '","InstrumentIdentifier":"' + sym + '","Periodicity":"' + prc + '","Period":' + str(
#                 prd) + ',"Max":' + str(rno) + ',"isShortIdentifier":"' + iss + '"}'
#         elif iss == "" and utg == "":
#             req_msg = '{"MessageType":"GetHistory","Exchange":"' + exg + '","InstrumentIdentifier":"' + sym + '","Periodicity":"' + prc + '","Period":' + str(
#                 prd) + ',"Max":' + str(rno) + ',"isShortIdentifier":"false"' + '"}'
#         elif iss == "" and utg != "":
#             req_msg = '{"MessageType":"GetHistory","Exchange":"' + exg + '","InstrumentIdentifier":"' + sym + '","Periodicity":"' + prc + '","Period":' + str(
#                 prd) + ',"Max":' + str(rno) + ',"isShortIdentifier":"false"' + '","UserTag":"' + utg + '"}'
#         await ws.send(str(req_msg))
#         print("Request : " + req_msg)
#         rmsg = await get_msg(ws)
#         return rmsg
#     except:
#         return "Exception : " + str(msg)
#
#
# async def get_msg(ws):
#     while True:
#         try:
#             message = await ws.recv()
#             rslt = json.loads(message)
#             if rslt["MessageType"] == 'HistoryOHLCResult' or rslt["MessageType"] == 'HistoryTickResult':
#                 return message
#         except websockets.ConnectionClosedOK:
#             break
#         # print(message)

import asyncio
import websockets
import json

# Function to get messages from WebSocket
async def get_msg(ws):
    while True:
        try:
            message = await ws.recv()
            rslt = json.loads(message)
            if rslt["MessageType"] == 'HistoryOHLCResult' or rslt["MessageType"] == 'HistoryTickResult':
                return message
        except websockets.ConnectionClosedOK:
            break


# Function to request historical data with from-time and to-time
async def mass_subscribe_n_streamft(ws, exg, sym, prc, prd, frm, to,max, iss, utg):
    req_msg = {
        "MessageType": "GetHistory",
        "Exchange": exg,
        "InstrumentIdentifier": sym,
        "Periodicity": prc,
        "Period": prd,
        "From": frm,
        "To": to,
        "Max":max,
        "isShortIdentifier": iss if iss else "false",
        "UserTag": utg if utg else ""

    }
    await ws.send(json.dumps(req_msg))
    print(req_msg)
    return await get_msg(ws)


# Function to request historical data with a limit on maximum records
async def mass_subscribe_n_stream(ws, exg, sym, prc, prd, iss, rno, utg):
    req_msg = {
        "MessageType": "GetHistory",
        "Exchange": exg,
        "InstrumentIdentifier": sym,
        "Periodicity": prc,
        "Period": prd,
        "Max": rno,
        "isShortIdentifier": iss if iss else "false",
        "UserTag": utg if utg else ""
    }
    await ws.send(json.dumps(req_msg))
    print(req_msg)
    return await get_msg(ws)

async def mass_subscribe_n_streamad(ws, exg, sym, prc, prd,frm,to,adj, iss, rno, utg):
    req_msg = {
        "MessageType": "GetHistory",
        "Exchange": exg,
        "InstrumentIdentifier": sym,
        "Periodicity": prc,
        "Period": prd,
        "From": frm,
        "To": to,
        "AdjustSplits":adj,
        "Max": rno,
        "isShortIdentifier": iss if iss else "false",
        "UserTag": utg if utg else ""
    }
    await ws.send(json.dumps(req_msg))
    print(req_msg)
    return await get_msg(ws)


# Wrapper function to call mass_subscribe_n_streamft based on period range
def getbyperiod(ws, exchange, InstrumentIdentifier, periodicity, period, fromtime, totime,Max, usertag, isShortIdentifiers):
    exg = exchange
    sym = InstrumentIdentifier
    prc = periodicity or 'MINUTE'
    prd = period or 1
    iss = isShortIdentifiers or 'false'
    utg = usertag or ''

    return asyncio.get_event_loop().run_until_complete(
        mass_subscribe_n_streamft(ws, exg, sym, prc, prd, fromtime, totime,Max, iss, utg))


# Wrapper function to call mass_subscribe_n_stream for maximum records
def getcaldle(ws, exchange, InstrumentIdentifier, periodicity, period, max, usertag, isShortIdentifiers):
    exg = exchange
    sym = InstrumentIdentifier
    prc = periodicity or 'MINUTE'
    prd = period or 1
    rno = max or 10
    iss = isShortIdentifiers or 'false'
    utg = usertag or ''

    return asyncio.get_event_loop().run_until_complete(
        mass_subscribe_n_stream(ws, exg, sym, prc, prd, iss, rno, utg))

# Wrapper function to call mass_subscribe_n_stream for maximum records
def getadjusted(ws, exchange, InstrumentIdentifier, periodicity, period,fromtime, totime, max, AdjustSplits, usertag, isShortIdentifiers):
    exg = exchange
    sym = InstrumentIdentifier
    prc = periodicity or 'MINUTE'
    prd = period or 1
    rno = max or 10
    iss = isShortIdentifiers or 'false'
    utg = usertag or ''

    return asyncio.get_event_loop().run_until_complete(
        mass_subscribe_n_streamad(ws, exg, sym, prc, prd,fromtime, totime,AdjustSplits, iss, rno, utg))



import asyncio
import websockets
from datetime import datetime
import json
import time
import errno

url = None
key = None
exg = None
sym = None
msg = None
res = None
message = None


def get(ws):
    rmsg = asyncio.get_event_loop().run_until_complete(mass_subscribe_n_stream(ws))
    return rmsg


async def mass_subscribe_n_stream(ws):
    try:
        req_msg = str('{"MessageType":"GetExchanges"}')
        await ws.send(req_msg)
        print("Request : " + req_msg)
        rmsg = await get_msg(ws)  # Listens for the tick data until market close
        return rmsg
    except:
        return msg


async def get_msg(ws):
    while True:
        try:
            message = await ws.recv()
            rslt = json.loads(message)
            if rslt["MessageType"] == 'ExchangesResult':
                return message
        except websockets.ConnectionClosedOK:
            break




import asyncio
import websockets
import json
import errno
import os

url = None
key = None
exg = None
prc = None
prd = None
ist = None
dfm = None
dto = None
ntr = None


def get(ws, exchange, periodicity, period, instrumentType=None, dfrom=None, dto=None, nonTraded=None):
    if exchange == "":
        print("Exchange is mandatory.")
    else:
        exg = exchange

    if periodicity == "":
        prc = ''
    else:
        prc = periodicity

    if period == "":
        prd = ''
    else:
        prd = period

    if instrumentType == "":
        ist = ''
    else:
        ist = instrumentType

    if dfrom == "":
        dfm = ''
    else:
        dfm = dfrom

    if nonTraded == "":
        ntr = ''
    else:
        ntr = nonTraded
    rmsg = asyncio.get_event_loop().run_until_complete(mass_subscribe_n_stream(ws, exg, prc, prd, ist, dfm, dto, ntr))
    return rmsg


async def mass_subscribe_n_stream(ws, exg, prc, prd, ist, dfm, dto, ntr):
    try:
        req_msg = '{"MessageType":"GetExchangeSnapshot","Exchange":"' + exg + '","periodicity":"' + prc + '","period":"' + prd + '"'
        if ist is not None:
            req_msg = req_msg + ',"instrumentType":"' + ist + '"'
        if dfm is not None:
            req_msg = req_msg + ',"From":"' + dfm + '"'
        if dto is not None:
            req_msg = req_msg + ',"To":"' + dto + '"'
        if ntr is not None:
            req_msg = req_msg + ',"nonTraded":"' + ntr + '"'
        req_msg = str(req_msg + '}')
        print("Request : " + req_msg)
        await ws.send(req_msg)
        rmsg = await get_msg(ws)
        return rmsg
    except:
        return 'Exception...' + str(os.error)



async def get_msg(ws):
    while True:
        try:
            message = await ws.recv()
            rslt = json.loads(message)
            if rslt["MessageType"] == 'ExchangeSnapshotResult':
                return message
        except websockets.ConnectionClosedOK:
            break
        print(message)



# import asyncio
# import websockets
# import json
# url = None
# key = None
# exg = None
# ist = None
#
#
# def get(ws,Exchange, InstrumentType=None):
#     exg = Exchange
#     ist = InstrumentType
#     rmsg = asyncio.get_event_loop().run_until_complete(mass_subscribe_n_stream(ws, exg, ist))
#     return rmsg
#
#
# async def mass_subscribe_n_stream(ws, exg, ist):
#     try:
#         req_msg = '{"MessageType":"GetProducts","Exchange":"' + exg
#         if ist is not None:
#             req_msg = req_msg + '",InstrumentType":"' + ist
#
#         req_msg = str(req_msg + '"}')
#         print("Request : " + req_msg)
#         await ws.send(req_msg)
#         rmsg = await get_msg(ws)
#         return rmsg
#     except:
#         return "Error"
#
#
# async def get_msg(ws):
#     while True:
#         try:
#             message = await ws.recv()
#             rslt = json.loads(message)
#             if rslt.get["MessageType"] == 'ProductsResult':
#                 return message
#         except websockets.ConnectionClosedOK:
#             break


import asyncio
import websockets
import json

# Define variables
url = None
key = None
exg = None
ist = None

# Function to get the message from the WebSocket
def get(ws, Exchange, InstrumentType=None):
    exg = Exchange
    ist = InstrumentType
    # Use asyncio to run the async mass_subscribe_n_stream function synchronously
    rmsg = asyncio.get_event_loop().run_until_complete(mass_subscribe_n_stream(ws, exg, ist))
    return rmsg

# Async function to send request and handle response
async def mass_subscribe_n_stream(ws, exg, ist):
    try:
        # Create request message as a dictionary
        req_msg = {
            "MessageType": "GetProducts",
            "Exchange": exg
        }

        # Add InstrumentType if it's provided
        if ist is not None:
            req_msg["InstrumentType"] = ist

        # Convert dictionary to JSON string
        req_msg_str = json.dumps(req_msg)
        print(f"Request: {req_msg_str}")

        # Send the request message to the WebSocket
        await ws.send(req_msg_str)

        # Await the response message from the WebSocket
        rmsg = await get_msg(ws)
        return rmsg

    except Exception as e:
        # Print and return an error message if an exception occurs
        print(f"Error occurred: {e}")
        return "Error"

# Async function to receive and process WebSocket messages
async def get_msg(ws):
    while True:
        try:
            # Wait for message from the WebSocket
            message = await ws.recv()

            # Parse the message as JSON
            rslt = json.loads(message)

            # Check for the 'ProductsResult' message type and return it
            if rslt.get("MessageType") == 'ProductsResult':
                return message

        # Break the loop when the connection is closed
        except websockets.ConnectionClosedOK:
            break



import asyncio
import websockets
import json
import errno
import os

url = None
key = None
exg = None
dth = None
printed_once = False


def get(ws, exchange, Count):
    if exchange == "":
        return "Exchange is mandatory."
    else:
        exg = exchange

    if Count == "":
        return "product is mandatory."

    else:
        prd = Count

    rmsg = asyncio.get_event_loop().run_until_complete(
        mass_subscribe_n_stream(ws, exg, prd))
    return rmsg


async def mass_subscribe_n_stream(ws, exg, prd):
    global printed_once
    try:
        req_msg = '{"MessageType":"GetTopGainersLosers",'
        if exg is not None:
            req_msg = req_msg + '"Exchange":"' + exg + '"'
        if prd is not None:
            req_msg = req_msg + ',"Count":"' + str(prd) + '"'
        req_msg = str(req_msg + '}')
        if not printed_once:
            print("Request : " + req_msg)
            printed_once = True
        await ws.send(req_msg)
        rmsg = await get_msg(ws)
        return rmsg
    except:
        return 'In Exception...' + os.error


async def get_msg(ws):
    while True:
        try:
            message = await ws.recv()
            rslt = json.loads(message)
            if rslt["MessageType"] == 'LastQuoteArrayResult':
                return message
        except websockets.ConnectionClosedOK:
            break



import asyncio
import websockets
import json
import errno
import os

url = None
key = None
exg = None


def get(ws, exchange, instrumenttype=None, product=None):
    if exchange == "":
        print("Exchange is mandatory.")
    else:
        exg = exchange

    rmsg = asyncio.get_event_loop().run_until_complete(
        mass_subscribe_n_stream(ws, exg,instrumenttype, product))
    return rmsg


async def mass_subscribe_n_stream(ws, exg, ist, prd):
    try:
        req_msg = '{"MessageType":"GetExpiryDates","Exchange":"' + exg + '"'
        if ist is not None:
            req_msg = req_msg + '"InstrumentType":"' + ist + '"'
        if prd is not None:
            req_msg = req_msg + '"Product":"' + prd + '"'
        req_msg = str(req_msg + '}')
        print("Request : " + req_msg)
        await ws.send(req_msg)
        rmsg = await get_msg(ws)
        return rmsg
    except:
        return 'In Exception...' + os.error


async def get_msg(ws):
    while True:
        try:
            message = await ws.recv()
            rslt = json.loads(message)
            if rslt["MessageType"] == 'ExpiryDatesResult':
                return message
        except websockets.ConnectionClosedOK:
            break
        print(message)



import asyncio
import websockets
import json

exg = None
sym = None
msg = None
isi = None


def get(ws, exchange, symbols,isShortIdentifiers):
    exg = exchange
    sym = symbols
    isi = isShortIdentifiers
    rmsg = asyncio.get_event_loop().run_until_complete(mass_subscribe_n_stream(ws, exg, sym, isi))
    return rmsg


async def mass_subscribe_n_stream(ws, exg, sym, isi):
    try:
        instrument_identifiers = [{"Value": s} for s in sym.split(',')]

        req_msg = {
            "MessageType": "GetLastQuoteArrayShort",
            "Exchange": exg,
            "isShortIdentifiers": isi,
            "InstrumentIdentifiers": instrument_identifiers
        }
        req_msg_str = json.dumps(req_msg)
        print("Request : " + req_msg_str)
        await ws.send(req_msg_str)
        rmsg = await get_msg(ws)
        return rmsg
    except Exception as e:
        print(f"An error occurred: {e}")
        return msg


async def get_msg(ws):
    while True:
        try:
            message = await ws.recv()
            rslt = json.loads(message)
            if rslt["MessageType"] == 'LastQuoteArrayShortResult':
                return message
        except websockets.ConnectionClosedOK:
            break


import asyncio
import websockets
import json

url = None
key = None
exg = None
sym = None
msg = None
ity = None

exp = None
otp = None
srp = None
oa = None
dth =None
printed_once = False

def get(ws, exchange,product,expiry, strikeprice,depth,optiontype=None, Unsubscribe=None):
    exg = exchange
    sym = product
    exp= expiry
    srp = strikeprice
    ity = optiontype
    dth =depth
    
    rmsg = asyncio.get_event_loop().run_until_complete(mass_subscribe_n_stream(ws, exg, sym,exp,srp,ity,dth))
    return rmsg


def stop(ws, exchange, product,expiry, strikeprice,depth,optiontype=None, Unsubscribe=None):
    exg = exchange
    sym = product
    exp= expiry
    srp = strikeprice
    ity = optiontype
    dth =depth
    rmsg = asyncio.get_event_loop().run_until_complete(mass_unsubscribe_n_stream(ws, exg, sym,exp,srp,ity,dth))
    return rmsg


async def mass_subscribe_n_stream(ws, exg, sym,exp,srp,ity,dth):
    global  printed_once
    req_msg = str(
        '{"MessageType":"SubscribeOptionChainGreeks","Exchange":"' + exg + '","Unsubscribe":"false","Product":"' + sym + '","Expiry":"' + exp + '","StrikePrice":"' + srp + '","OptionType":"' + ity + '","Depth":"' + dth + '"}')
    await ws.send(req_msg)
    if not printed_once:
        print("Request : " + req_msg)
        printed_once = True
    rmsg = await get_msg(ws)
    return rmsg


async def mass_unsubscribe_n_stream(ws, exg, sym,exp,srp,ity,dth):
    global printed_once
    req_msg = str(
        '{"MessageType":"SubscribeOptionChainGreeks","Exchange":"' + exg + '","Unsubscribe":"true","Product":"' + sym + '","Expiry":"' + exp + '","StrikePrice":"' + srp + '","OptionType":"' + ity + '","Depth":"' + dth + '"}')
    if not printed_once:
        print("Request : " + req_msg)
        printed_once = True
    await ws.send(req_msg)
    rmsg = await get_msg(ws)
    return rmsg


async def get_msg(ws):
    while True:
        try:
            message = await ws.recv()
            rslt = json.loads(message)
            if rslt["MessageType"] == 'RealtimeOptionChainGreeksResult':
                return message
        except websockets.ConnectionClosedOK:
            break



import asyncio
import websockets
import json

url = None
key = None
exg = None
sym = None
msg = None
prc = None
prd = None
iss = None
printed_once = False

def get(ws, exchange, symbol, periodicity, period, unsubscribe =None):
    exg = exchange
    sym = symbol
    prc = periodicity
    prd = period
    uns =unsubscribe or "false"
    rmsg = asyncio.get_event_loop().run_until_complete(mass_subscribe_n_stream(ws, exg, sym, prc, prd ,uns))
    return rmsg


async def mass_subscribe_n_stream(ws, exg, sym, prc, prd,uns):
    global printed_once
    try:
        req_msg = str(
            '{"MessageType":"SubscribeSnapshot","Exchange":"' + exg + '","Periodicity":"' + prc + '","Period":' + f'{prd}' + ',"InstrumentIdentifier": "' + sym + '","unsubscribe": "' + uns + '"}')
        if not printed_once:
            print("Request : " + req_msg)
            printed_once = True
        await ws.send(req_msg)
        rmsg = await get_msg(ws)  # Listens for the tick data until market close
        return rmsg
    except:
        return msg


async def get_msg(ws):
    while True:
        try:
            message = await ws.recv()
            rslt = json.loads(message)
            if rslt["MessageType"] == 'RealtimeSnapshotResult':
                return message
        except websockets.ConnectionClosedOK:
            break


from setuptools import setup, find_packages

VERSION = '1.3.3'

DESCRIPTION = 'Package to implement WebSocket API of Global Datafeeds'
LONG_DESCRIPTION = 'Package to implement WebSocket API of Global Datafeeds. This api will provide realtime data as ' \
                   'well as historical data. '

# Setting up
setup(
    # the name must match the folder name 'verysimplemodule'
    name="gfdlws",
    version=VERSION,
    author="Global Datafeeds",
    author_email="developer@globaldatafeeds.in",
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    readme="README.md",
    packages=find_packages(),
    install_requires=['websockets'],  # add any additional packages that
    # needs to be installed along with your package. Eg: 'caer'

    keywords=['python', 'first package'],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Education",
        "Programming Language :: Python :: 3.9",
        "Operating System :: Microsoft :: Windows",
    ]
)



import asyncio
import websockets
import json
import errno
import os

url = None
key = None
exg = None
ity = None
prd = None
exp = None
otp = None
srp = None
oa = None


def get(ws, exchange, instrumenttype=None, product=None, expiry=None, optiontype=None, strikeprice=None, onlyactive=None):
    if exchange == "":
        return "Exchange is mandatory."
    else:
        exg = exchange

    if instrumenttype == "":
        ity = ''
    else:
        ity = instrumenttype

    if product == "":
        prd = ''
    else:
        prd = product

    if expiry == "":
        exp = ''
    else:
        exp = expiry

    if optiontype == "":
        otp = ''
    else:
        otp = optiontype

    if strikeprice == "":
        srp = ''
    else:
        srp = strikeprice

    if onlyactive == "":
        oa = ''
    else:
        oa = onlyactive

    rmsg = asyncio.get_event_loop().run_until_complete(
        mass_subscribe_n_stream(ws, exg, ity, prd, exp, otp, srp, oa))
    return rmsg


async def mass_subscribe_n_stream(ws, exg, ity, prd, exp, otp, srp, oa):
    try:
        req_msg = '{"MessageType":"GetInstruments",'
        if exg is not None:
            req_msg = req_msg + '"Exchange":"' + exg + '"'
        if ity is not None:
            req_msg = req_msg + ',"InstrumentType":"' + ity + '"'
        if prd is not None:
            req_msg = req_msg + ',"Product":"' + prd + '"'
        if exp is not None:
            req_msg = req_msg + ',"Expiry":"' + exp + '"'
        if otp is not None:
            req_msg = req_msg + ',"optionType":"' + otp + '"'
        if srp is not None:
            req_msg = req_msg + ',"strikePrice":"' + srp + '"'
        if oa is not None:
            req_msg = req_msg + ',"onlyActive":"' + oa + '"'
        req_msg = str(req_msg + '}')
        print("Request : " + req_msg)
        await ws.send(req_msg)
        rmsg = await get_msg(ws)
        return rmsg
    except:
        return 'In Exception...' + os.error



async def get_msg(ws):
    while True:
        try:
            message = await ws.recv()
            rslt = json.loads(message)
            if rslt["MessageType"] == 'InstrumentsResult':
                return message
        except websockets.ConnectionClosedOK:
            break
            return message


import asyncio
import websockets
import json


def get(ws):
    rmsg = asyncio.get_event_loop().run_until_complete(
        mass_subscribe_n_stream(ws))
    return rmsg


async def mass_subscribe_n_stream(ws):
    try:
        req_msg = str('{"MessageType":"GetServerInfo"}')
        print("Request : " + req_msg)
        await ws.send(req_msg)
        rmsg = await get_msg(ws)
        return rmsg
    except:
        return "Error"


async def get_msg(ws):
    while True:
        try:
            message = await ws.recv()
            rslt = json.loads(message)
            if rslt["MessageType"] == 'ServerInfoResult':
                return message
        except websockets.ConnectionClosedOK:
            break



import asyncio
import websockets
import json
import errno
import os

url = None
key = None
exg = None
prd = None
exp = None
otp = None
srp = None
oa = None

printed_once = False


def get(ws, exchange, InstrumentIdentifier, isShortIdentifier, max=None, From=None, To=None):
    if exchange == "":
        return "Exchange is mandatory."
    else:
        exg = exchange

    if InstrumentIdentifier == "":
        return "InstrumentIdentifier is mandatory."

    else:
        prd = InstrumentIdentifier

    if isShortIdentifier == "":
        return "isShortIdentifier is mandatory."

    else:
        exp = isShortIdentifier

    if max == "":
        otp = ''
    else:
        otp = max
    if From == "":
        srp = ''
    else:
        srp = From
    if To == "":
        oa = ''
    else:
        oa = To


    rmsg = asyncio.get_event_loop().run_until_complete(
        mass_subscribe_n_stream(ws, exg, prd, exp, otp, srp, oa))
    return rmsg


async def mass_subscribe_n_stream(ws, exg, prd, exp, otp, srp, oa):
    global printed_once
    try:
        req_msg = '{"MessageType":"GetHistoryGreeks",'
        if exg is not None:
            req_msg = req_msg + '"Exchange":"' + exg + '"'
        if prd is not None:
            req_msg = req_msg + ',"InstrumentIdentifier":"' + prd + '"'
        if exp is not None:
            req_msg = req_msg + ',"isShortIdentifier":"' + exp + '"'
        if otp is not None:
            req_msg = req_msg + ',"Max":"' +str(otp)  + '"'
        if srp is not None:
            req_msg = req_msg + ',"From":"' + str(srp) + '"'
        if oa is not None:
            req_msg = req_msg + ',"To":"' + str(oa) + '"'
        req_msg = str(req_msg + '}')
        if not printed_once:
            print("Request : " + req_msg)
            printed_once = True
        await ws.send(req_msg)
        rmsg = await get_msg(ws)
        return rmsg
    except:
        return 'In Exception...' + os.error


async def get_msg(ws):
    while True:
        try:
            message = await ws.recv()
            rslt = json.loads(message)
            if rslt["MessageType"] == 'HistoryGreeksResult':
                return message
        except websockets.ConnectionClosedOK:
            break
